@@ -2,14 +2,43 @@
 #define _DATATYPES_H_
 
 #include <stddef.h>
+#include <stdbool.h>
 #include "imconfig.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*
+=item MAXCHANNELS
+
+Maximum number of total color/alpha channels.
+
+=cut
+*/
+
 #define MAXCHANNELS 4
 
+/*
+=item MAXEXTRACHANNELS
+
+Maximum number of extra channels.
+
+=cut
+*/
+
+#define MAXEXTRACHANNELS 10
+
+/*
+=item MAXTOTALCHANNELS
+
+Maximum of total channels, including color, alpha and extra channels.
+
+=cut
+*/
+
+#define MAXTOTALCHANNELS 14
+
 /*
 =item im_context_t
 =category Data Types
@@ -173,14 +202,124 @@ typedef enum {
   /* a paletted image might have one bit per sample */
   i_8_bits = 8,
   i_16_bits = 16,
-  i_double_bits = sizeof(double) * 8
+  i_double_bits = sizeof(double) * 8,
+
+  /* future use, defined so the underlying integral type doesn't change later */
+  i_32_bits = 32,
+  i_float16_bits = -16,
+  i_float_bits = -(int)(sizeof(float)*8)
 } i_img_bits_t;
 
+/*
+=item i_data_layout_t
+
+Used in calls to i_img_data() to request a particular layout of samples.
+
+The numeric values 1-4 correspond to the Imager default layouts for
+direct images.
+
+See L<Imager::APIRef/i_img_data()> for details.
+
+=cut
+*/
+
+typedef enum {
+  idl_palette,
+  idl_gray,
+  idl_gray_alpha,
+  idl_rgb,
+  idl_rgb_alpha,
+
+  idl_rgbx,
+  idl_bgr,
+  idl_abgr,
+} i_data_layout_t;
+
+/*
+=item i_data_flags_t
+
+Flags supplied to i_img_data() to control behavior:
+
+=over
+
+=item *
+
+C<idf_synthesize> - synthesize the image data if needed.  Data cannot
+be synthesized if C<idf_writable> is also set.
+
+=item *
+
+C<idf_writable> - the returned pointer can be used to write to the
+image.
+
+=item *
+
+C<idf_extras> - image data returned may include channels after the
+channels implied by the data layout.
+
+=item *
+
+C<idf_bigendian> - return the data in big endian byte order.
+
+=item *
+
+C<idf_littleendian> - return the data in little endian byte order.
+
+=item *
+
+C<idf_otherendian> - return the data in the opposite to the native
+byte order.  Either C<idf_bigendia> or C<idf_littleendian> will equal
+this, while the other will be zero.
+
+=item *
+
+C<idf_linear_curve> - return the data with samples that have a linear
+relationship to luminance, rather than following the tone curve of the
+current color profile.
+
+=back
+
+=cut
+*/
+
+typedef enum {
+  idf_synthesize    = 0x1,
+  idf_writable      = 0x2,
+  idf_extras        = 0x4,
+  idf_otherendian   = 0x8,
+#if defined IMAGER_LITTLE_ENDIAN
+  idf_littleendian  = 0,
+  idf_bigendian     = idf_otherendian,
+#elif defined IMAGER_BIG_ENDIAN
+  idf_littleendian  = idf_otherendian,
+  idf_bigendian     = 0,
+#else
+#  error No endianess defined
+#endif
+  idf_linear_curve  = 0x10,
+  /* so extra flags don't modify the size of the type */
+  idf_reserved      = 0x80000000
+} i_data_flags_t;
+
+/*
+=item i_image_data_alloc_t
+
+i_img_data() returns a pointer to this mostly opaque type.
+
+This must be released with i_img_data_release().
+
+=cut
+*/
+
+typedef struct i_image_data_alloc_struct {
+  void (*f_release)(struct i_image_data_alloc_struct *p);
+} i_image_data_alloc_t;
+
 typedef struct {
-  char *name; /* name of a given tag, might be NULL */
-  int code; /* number of a given tag, -1 if it has no meaning */
+  char *name; /* name of a given tag */
+  int code; /* number of a given tag, deprecated */
   char *data; /* value of a given tag if it's not an int, may be NULL */
-  int size; /* size of the data */
+  size_t size; /* size of the data */
   int idata; /* value of a given tag if data is NULL */
 } i_img_tag;
 
@@ -191,15 +330,6 @@ typedef struct {
 } i_img_tags;
 
 typedef struct i_img_ i_img;
-typedef int (*i_f_ppix_t)(i_img *im, i_img_dim x, i_img_dim y, const i_color *pix);
-typedef int (*i_f_ppixf_t)(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *pix);
-typedef i_img_dim (*i_f_plin_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, const i_color *vals);
-typedef i_img_dim (*i_f_plinf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, const i_fcolor *vals);
-typedef int (*i_f_gpix_t)(i_img *im, i_img_dim x, i_img_dim y, i_color *pix);
-typedef int (*i_f_gpixf_t)(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *pix);
-typedef i_img_dim (*i_f_glin_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_color *vals);
-typedef i_img_dim (*i_f_glinf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_fcolor *vals);
-
 typedef i_img_dim (*i_f_gsamp_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_sample_t *samp,
                            const int *chans, int chan_count);
 typedef i_img_dim (*i_f_gsampf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_fsample_t *samp,
@@ -228,6 +358,132 @@ typedef i_img_dim
 (*i_f_psampf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
 		const i_fsample_t *samp, const int *chan, int chan_count);
 
+typedef i_image_data_alloc_t *
+(*i_f_data_t)(i_img *im, i_data_layout_t layout, i_img_bits_t bits, unsigned flags,
+              void **p, size_t *size, int *extra);
+
+typedef int
+(*i_f_imageop_t)(i_img *im, const char *opname, void *params);
+
+typedef i_img_dim
+(*i_f_gslin_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+	       i_sample16_t *samp, const int *chan, int chan_count);
+
+typedef i_img_dim
+(*i_f_gslinf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+		i_fsample_t *samp, const int *chan, int chan_count);
+
+typedef i_img_dim
+(*i_f_pslin_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+	       const i_sample16_t *samp, const int *chan, int chan_count);
+
+typedef i_img_dim
+(*i_f_pslinf_t)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y,
+		const i_fsample_t *samp, const int *chan, int chan_count);
+
+/*
+=item i_img_vtable
+=category Data Types
+
+New image entry-points go in here rather than making i_img larger.
+
+Note that access to entries in here is controlled by IMAGER_API_LEVEL.
+
+Elements:
+
+=over
+
+=item *
+
+i_f_gsamp, i_f_gampf - implementations for each
+of the required image functions.  An image implementation should
+initialize these between calling i_img_alloc() and i_img_init().
+
+=item *
+
+i_f_psamp - implements psamp() for this image.
+
+=item *
+
+i_f_psampf - implements psamp() for this image.
+
+=item *
+
+i_f_destroy - custom image destruction function.  This should be used
+to release memory if necessary.
+
+=item *
+
+i_f_gsamp_bits - implements i_gsamp_bits() for this image.
+
+=item *
+
+i_f_psamp_bits - implements i_psamp_bits() for this image.
+
+=item *
+
+i_f_gslin, i_f_gslinf - implements i_gslin() and i_gslinf().
+
+=item *
+
+i_f_pslin, i_f_pslinf - implements i_pslin() and i_pslinf().
+
+=item *
+
+i_f_data - implements i_img_data() for this image.
+
+=item *
+
+i_f_imageop - future use, I hope.
+
+=item *
+
+i_f_gpal, i_f_ppal, i_f_addcolors, i_f_getcolors, i_f_colorcount,
+i_f_maxcolors, i_f_findcolor, i_f_setcolors - implementations for each
+paletted image function.
+
+=back
+
+=cut
+*/
+
+typedef struct i_img_vtable_struct {
+  int api_level;
+
+  /* interface functions */
+  i_f_gsamp_t i_f_gsamp;
+  i_f_gsampf_t i_f_gsampf;
+  
+  /* as of 0.88 */
+  i_f_psamp_t i_f_psamp;
+  i_f_psampf_t i_f_psampf;
+
+  i_f_destroy_t i_f_destroy;
+
+  /* as of 0.61 */
+  i_f_gsamp_bits_t i_f_gsamp_bits;
+  i_f_psamp_bits_t i_f_psamp_bits;
+
+  i_f_gslin_t i_f_gslin;
+  i_f_gslinf_t i_f_gslinf;
+  i_f_pslin_t i_f_pslin;
+  i_f_pslinf_t i_f_pslinf;
+
+  i_f_data_t i_f_data;
+
+  i_f_imageop_t i_f_imageop;
+
+  /* only valid for type == i_palette_type */
+  i_f_gpal_t i_f_gpal;
+  i_f_ppal_t i_f_ppal;
+  i_f_addcolors_t i_f_addcolors;
+  i_f_getcolors_t i_f_getcolors;
+  i_f_colorcount_t i_f_colorcount;
+  i_f_maxcolors_t i_f_maxcolors;
+  i_f_findcolor_t i_f_findcolor;
+  i_f_setcolors_t i_f_setcolors;
+} i_img_vtable;
+
 /*
 =item i_img
 =category Data Types
@@ -242,7 +498,13 @@ It contains the following members:
 
 =item *
 
-C<channels> - the number of channels in the image
+C<channels> - the number of channels in the image.  This is only the
+channels associated with color and alpha channel.  See also C<extrachannels>.
+
+=item *
+
+C<extrachannels> - the number of extra channels associated with each
+pixel.
 
 =item *
 
@@ -271,14 +533,23 @@ for paletted images.
 
 =item *
 
+=for stopwords natively
+
+C<islinear> - whether the image samples are natively linear or use a
+tone curve.  Accessor: i_img_linear().
+
+=item *
+
 C<isvirtual> - if zero then this image is-self contained.  If non-zero
 then this image could be an interface to some other implementation.
 
 =item *
 
-C<idata> - the image data.  This should not be directly accessed.  A new
-image implementation can use this to store its image data.
-i_img_destroy() will myfree() this pointer if it's non-null.
+C<idata> - the image data.  This should not be directly accessed.  A
+new image implementation can use this to store its image data.
+L</i_img_destroy()> will myfree() this pointer if it's non-null.
+Non-image implementors can use L</i_img_data()> to fetch or optionally
+synthesize raw image data.
 
 =item *
 
@@ -297,44 +568,6 @@ i_img_init().
 
 =item *
 
-i_f_ppix, i_f_ppixf, i_f_plin, i_f_plinf, i_f_gpix, i_f_gpixf,
-i_f_glin, i_f_glinf, i_f_gsamp, i_f_gampf - implementations for each
-of the required image functions.  An image implementation should
-initialize these between calling i_img_alloc() and i_img_init().
-
-=item *
-
-i_f_gpal, i_f_ppal, i_f_addcolors, i_f_getcolors, i_f_colorcount,
-i_f_maxcolors, i_f_findcolor, i_f_setcolors - implementations for each
-paletted image function.
-
-=item *
-
-i_f_destroy - custom image destruction function.  This should be used
-to release memory if necessary.
-
-=item *
-
-i_f_gsamp_bits - implements i_gsamp_bits() for this image.
-
-=item *
-
-i_f_psamp_bits - implements i_psamp_bits() for this image.
-
-=item *
-
-i_f_psamp - implements psamp() for this image.
-
-=item *
-
-i_f_psampf - implements psamp() for this image.
-
-=item *
-
-C<im_data> - image specific data internal to Imager.
-
-=item *
-
 C<context> - the Imager API context this image belongs to.
 
 =back
@@ -343,51 +576,24 @@ C<context> - the Imager API context this image belongs to.
 */
 
 struct i_img_ {
+  const i_img_vtable *vtbl;
   int channels;
+  int extrachannels;
   i_img_dim xsize,ysize;
   size_t bytes;
   unsigned int ch_mask;
   i_img_bits_t bits;
   i_img_type_t type;
+  int islinear;
   int isvirtual; /* image might not keep any data, must use functions */
   unsigned char *idata; /* renamed to force inspection of existing code */
                         /* can be NULL if isvirtual is non-zero */
   i_img_tags tags;
 
+  unsigned ref_count;
+
   void *ext_data;
 
-  /* interface functions */
-  i_f_ppix_t i_f_ppix;
-  i_f_ppixf_t i_f_ppixf;
-  i_f_plin_t i_f_plin;
-  i_f_plinf_t i_f_plinf;
-  i_f_gpix_t i_f_gpix;
-  i_f_gpixf_t i_f_gpixf;
-  i_f_glin_t i_f_glin;
-  i_f_glinf_t i_f_glinf;
-  i_f_gsamp_t i_f_gsamp;
-  i_f_gsampf_t i_f_gsampf;
-  
-  /* only valid for type == i_palette_type */
-  i_f_gpal_t i_f_gpal;
-  i_f_ppal_t i_f_ppal;
-  i_f_addcolors_t i_f_addcolors;
-  i_f_getcolors_t i_f_getcolors;
-  i_f_colorcount_t i_f_colorcount;
-  i_f_maxcolors_t i_f_maxcolors;
-  i_f_findcolor_t i_f_findcolor;
-  i_f_setcolors_t i_f_setcolors;
-
-  i_f_destroy_t i_f_destroy;
-
-  /* as of 0.61 */
-  i_f_gsamp_bits_t i_f_gsamp_bits;
-  i_f_psamp_bits_t i_f_psamp_bits;
-
-  /* as of 0.88 */
-  i_f_psamp_t i_f_psamp;
-  i_f_psampf_t i_f_psampf;
-
   void *im_data;
 
   /* 0.91 */
@@ -1139,6 +1345,8 @@ typedef enum {
   icm_rgb_alpha
 } i_color_model_t;
 
+#include "imcmst.h"
+
 #ifdef IMAGER_FORMAT_ATTR
 #define I_FORMAT_ATTR(format_index, va_index) \
   __attribute ((format (printf, format_index, va_index)))
